import os
import pygame
from pygame.rect import Rect
from pygame.locals import *

from data import get_path

class Group (object):
	def __init__(self, dispatcher):
		self.dispatcher = dispatcher
		self.widgets = []
		self.enabled = True
	
	def add_widget(self, w):
		self.dispatcher.add_widget(w)
		self.widgets.append(w)
	
	def add_widgets(self, *w):
		self.dispatcher.add_widgets(*w)
		self.widgets.extend(w)
	
	def enable(self):
		if not self.enabled:
			self.dispatcher.add_widgets(*self.widgets)
			self.enabled = True

	def disable(self):
		if self.enabled:
			self.dispatcher.del_widgets(*self.widgets)
			self.enabled = False


class Widget (object):
	padding = (4, 1)
	fg_color = (255, 255, 255)
	bg_color = (0, 0, 0)
	hl_color = (64, 64, 64)
	fontname = None
	fontsize = None

	def __init__(self, rect):
		self.rect = Rect(rect)

	def draw(self, surf):
		pass

	def focus(self):
		pass

	def blur(self):
		pass

	def clicked(self, button, pos):
		pass
	
	def motion(self, buttons, pos, rel):
		"""Motion event is passed to the widget over which cursor is right now."""
		pass

	def dragging(self, button, start, pos, rel):
		"""Dragging event is passed to the widget over which mouse button was initialy pressed."""
		pass
	
	def dragged(self, button, start, pos):
		"""When drag finished."""
		pass
	
class Box (Widget):
	def draw(self, surf):
		if self.bg_color:
			pygame.draw.rect(surf, self.bg_color, self.rect)

		pygame.draw.rect(surf, self.fg_color, self.rect, 1)

class Label (Widget):
	def __init__(self, rect, text):
		Widget.__init__(self, rect)

		self.textpos = self.rect.move(self.padding)
		self.text = text
	
	def update(self, *params):
		self.surf = render_text(self.fontname, self.fontsize, self.text % params, self.fg_color)
	
	def draw(self, surf):
		surf.blit(self.surf, self.textpos)

class ActiveBox (Box):

	def __init__(self, rect):
		Box.__init__(self, rect)
		self.norm_color = self.bg_color

	def focus(self):
		self.bg_color = self.hl_color

	def blur(self):
		self.bg_color = self.norm_color

class Button (ActiveBox):
	def __init__(self, rect, text, fontname=None, fontsize=None):
		ActiveBox.__init__(self, rect)

		self.text = text
		self.textpos = self.rect.move(self.padding)

		if not fontname:
			fontname = self.fontname
		if not fontsize:
			fontsize = self.fontsize

		self.surf = render_text(fontname, fontsize, text, self.fg_color)
	
	def draw(self, surf):
		ActiveBox.draw(self, surf)
		surf.blit(self.surf, self.textpos)

class ProgressButton (Widget):
	def __init__(self, rect, text_normal, text_hover):
		Widget.__init__(self, rect)
		self.text_normal = text_normal
		self.text_hover = text_hover

		self.surf_normal = render_text(self.fontname, self.fontsize, text_normal, self.fg_color)
		self.surf_hover = render_text(self.fontname, self.fontsize, text_hover, self.fg_color)
		self.surf = self.surf_normal

		self.update(0)

		self.textpos = self.rect.move(self.padding)
	
	def focus(self):
		self.surf = self.surf_hover

	def blur(self):
		self.surf = self.surf_normal

	def on_draw(self):
		pass

	def update(self, value):
		"""Update progress bar.

		Values are flaots from zero to one, inclusive.
		"""
		self.progress = self.rect.move(0, 0)
		self.progress.w = int(self.rect.w * value)

	def draw(self, surf):
		self.on_draw()
		surf.set_clip(self.rect)
		surf.fill(self.bg_color)
		surf.set_clip(self.progress)
		surf.fill(self.hl_color)
		surf.set_clip()
		pygame.draw.rect(surf, self.fg_color, self.rect, 1)
		surf.blit(self.surf, self.textpos)

class Dispatcher (object):
	"""Passes events to widgets.
	"""

	click_threshold = 5

	def __init__(self):
		self.widgets = {}

		self.focused = None
		self.clicked = {}
		self.default = None

	def find_widget(self, pos):
		p = Rect(pos[0], pos[1], 1, 1)
		w = p.collidedict(self.widgets)
		if w:
			return w[1]
		else:
			return self.default
	
	def update_focus(self, pos):
		self.set_focus(self.find_widget(pos))
	
	def set_focus(self, w):
		if w == self.focused:
			return

		if self.focused:
			self.focused.blur()

		self.focused = w
		if w:
			w.focus()

	def handle_event(self, ev):
		if ev.type == MOUSEBUTTONDOWN:
			w = self.find_widget(ev.pos)
			if w:
				self.clicked[ev.button] = (ev.pos, w)

		elif ev.type == MOUSEBUTTONUP:
			prev, w = self.clicked.get(ev.button, (None, None))
			if prev:
				del self.clicked[ev.button]
				dx = ev.pos[0] - prev[0]
				dy = ev.pos[1] - prev[1]
				if dx**2 + dy**2 < self.click_threshold:
					w.clicked(ev.button, ev.pos)
				else:
					w.dragged(ev.button, prev, ev.pos)

		elif ev.type == MOUSEMOTION:
			self.update_focus(ev.pos)
			if self.focused:
				self.focused.motion(ev.buttons, ev.pos, ev.rel)

			for button in self.clicked.keys():
				if ev.buttons[button - 1]:
					prev, w = self.clicked[button]
					w.dragging(button, prev, ev.pos, ev.rel)

	def add_widget(self, w):
		self.widgets[tuple(w.rect)] = w
		return w

	def add_widgets(self, *widgets):
		for w in widgets:
			self.add_widget(w)
	
	def del_widget(self, w):
		del self.widgets[tuple(w.rect)]

	def del_widgets(self, *widgets):
		for w in widgets:
			self.del_widget(w)

	def draw_all(self, surf):
		for w in self.widgets.values():
			w.draw(surf)

font_cache = {}

default_fontname = 'DejaVuSans.ttf'
default_fontsize = 16
def render_text(name, size, text, color):
	font = font_cache.get((name, size))
	if not font:
		if not name:
			name = default_fontname

		if not size:
			size = default_fontsize

		name = get_path(os.path.join('fonts', name))
		font = pygame.font.Font(name, size)
		font_cache[(name, size)] = font

	return font.render(text, 1, color)

if __name__ == '__main__':
	pygame.init()
	disp = pygame.display.set_mode((1024, 768))

	Widget.bg_color = (32, 32, 64)
	Widget.font = pygame.font.Font(pygame.font.get_default_font(), 14)
	ActiveBox.hl_color = (64, 128, 128)

	class TestButton (Button):
		def clicked(self, button, pos):
			print 'CLICK', self.text, pos

	dispatcher = Dispatcher()
	dispatcher.add_widget(TestButton((10, 10, 150, 20), 'Test button'))
	dispatcher.add_widget(TestButton((10, 34, 150, 20), 'Another one'))
	dispatcher.add_widget(TestButton((10, 58, 150, 20), 'And another'))

	running = 1
	while running:
		for ev in pygame.event.get():
			if ev.type == KEYDOWN and ev.key == K_ESCAPE or ev.type == QUIT:
				running = 0
			dispatcher.handle_event(ev)
			

		dispatcher.update_focus(pygame.mouse.get_pos())
		
		disp.fill((64, 64, 128))
		dispatcher.draw_all(disp)

		pygame.display.update()


